LIBRARY "custom.brs"
LIBRARY "helpers.brs"
LIBRARY "eventLoop.brs"

Sub Main()
    print "Main sub begin"
    msgPort = CreateObject("roMessagePort")

    systemTime = CreateObject("roSystemTime")
    firstStartSection = CreateObject("roRegistrySection", "firstStart")
    if not(firstStartSection.Exists("time")) then
        print "This is first start"
        currentLocalTime = systemTime.GetLocalDateTime()
        firstStartSection.Write("time", currentLocalTime.GetString())
        OnFirstStartHook()
    endif
    OnEveryStartHook()

    audioOutput = CreateObject("roAssociativeArray")
    audioOutput["HDMI"] = CreateObject("roAudioOutput", "HDMI")
    audioOutput["Analog"] = CreateObject("roAudioOutput", "Analog")
    audioOutput["SPDIF"] = CreateObject("roAudioOutput", "SPDIF")
    audioOutput["USB"] = CreateObject("roAudioOutput", "USB")
    audioOutput["NONE"] = CreateObject("roAudioOutput", "NONE")

    debugRegistrySection = CreateObject("roRegistrySection", "debug")
    debugPort = 3000

    registrySection = CreateObject("roRegistrySection", "html")
    multiProcess = registrySection.read("mp")
    if multiProcess <> "1" then
        registrySection.write("mp","1")
        registrySection.flush()
        RebootSystem()
    endif

    videoPlayers = CreateObject("roAssociativeArray")
    videoMetadata = CreateObject("roAssociativeArray")
    videoModeProvider = CreateObject("roVideoMode")
    videoModeProvider.SetGraphicsZOrder("back")
    videoModeProvider.SetBackgroundColor(&h18182c)
    screenRegistrySection = CreateObject("roRegistrySection", "screen")
    supportedVideoModes = GetSupportedVideoModes(videoModeProvider)
    print "supportedVideoModes";FormatJson(supportedVideoModes, 1)
    InitVideoMode(screenRegistrySection, supportedVideoModes, "800", "600")

    deviceInfo = CreateObject("roDeviceInfo")
    model = deviceInfo.GetModel()
    print "model=";model
    hsRegex = CreateObject("roRegex", "^hs\w+$", "i")
    if hsRegex.IsMatch(model) then
        print "HSxxxx series"
        defaultResolution = { width: 1280, height: 800 }
        InitVideoMode(screenRegistrySection, "1280", "800", "60")
    else
        print "Any specific model hasn't been recognized"
        defaultResolution = { width: 800, height: 600 }
        InitVideoMode(screenRegistrySection, "800", "600", "auto")
    endif

    fullScreenRectangle = CreateObject("roRectangle", 0, 0, defaultResolution.width, defaultResolution.height)
    touchScreen = CreateObject("roTouchScreen")
    touchScreen.SetPort(msgPort)
    touchScreen.SetResolution(defaultResolution.width, defaultResolution.height)
    ApplySavedVideoMode(screenRegistrySection, videoModeProvider, fullScreenRectangle, touchScreen)

    print "videoMode=";screenRegistrySection.Read("video_mode")
    print "orientation=";screenRegistrySection.Read("orientation")
    print "video orientation=";screenRegistrySection.Read("video_orientation")

    currentTransform = GetCurrentTransform(screenRegistrySection)
    config = {
        nodejs_enabled: true,
        brightsign_js_objects_enabled: true,
        mouse_enabled: true,
        focus_enabled: true,
        javascript_enabled: true,
        url: "file:///index.html",
        storage_path: "SD:",
        storage_quota: 1073741824,
        transform: currentTransform,
        hwz_default: "on",
        security_params: {
            websecurity: false
        }
        'port: msgPort ' Doesn't work in 6.2 FW. Used this htmlWidget.SetPort(msgPort)
    }

    if debugRegistrySection.Read("native") = "1" then
        config["inspector_server"] = {
            port: debugPort
        }

        networkingRegistry = CreateObject("roRegistrySection", "networking")
        networkingRegistry.write("ssh","22")
        networkingRegistry.write("telnet","23")
        networkingRegistry.flush()
    endif

    htmlWidget = CreateObject("roHtmlWidget", fullScreenRectangle, config)
    htmlWidget.SetPort(msgPort)
    htmlWidget.SetProxy("")
    sleep(5000)
    htmlWidget.Show()

    serialPorts = CreateObject("roAssociativeArray")

    htmlWidget.PostJSMessage({
        type: "ready"
    })

    while true
        msg = Wait(0, msgPort)
        if type(msg) = "roHtmlWidgetEvent" then
            ProcessHtmlWidgetEvent(msg, msgPort, htmlWidget, videoModeProvider, supportedVideoModes, audioOutput, systemTime, screenRegistrySection, debugRegistrySection, videoPlayers, videoMetadata, touchScreen, serialPorts)
        else if type(msg) = "roTouchEvent" then
            ProcessTouchEvent(msg, htmlWidget, screenRegistrySection)
        else if type(msg) = "roVideoEvent" then
            ProcessVideoEvent(msg, htmlWidget, videoPlayers, videoMetadata)
        else if type(msg) = "roStreamLineEvent" then
            ProcessHardwareEvent(msg.GetString(), htmlWidget, serialPorts)
        else if type(msg) = "roStreamByteEvent" then
            ProcessHardwareEvent(msg.GetInt(), htmlWidget, serialPorts)
        endif
    end while

End Sub
