Sub ProcessTouchEvent(msg, htmlWidget, screenRegistrySection)
    orientation = screenRegistrySection.Read("orientation")
    resolution = GetCurrentResolution(screenRegistrySection)
    touchEvent = CreateObject("roAssociativeArray")
    touchEvent["type"] = "touch_event"
    if orientation = "PORTRAIT" then
        touchEvent["x"] = resolution.height - msg.GetY()
        touchEvent["y"] = msg.GetX()
    else if orientation = "PORTRAIT_FLIPPED" then
        touchEvent["x"] = msg.GetY()
        touchEvent["y"] = resolution.width - msg.GetX()
    else
        touchEvent["x"] = msg.GetX()
        touchEvent["y"] = msg.GetY()
    endif 
    htmlWidget.PostJSMessage(touchEvent)
End Sub

Sub ProcessVideoEvent(msg, htmlWidget, videoPlayers, videoMetadata)
    videoEvent = CreateObject("roAssociativeArray")
    videoEvent["type"] = "video_event"
    videoEvent["key"] = msg.GetUserData()
    videoEvent["eventType"] = msg.GetInt()
    videoPlayer = videoPlayers.Lookup(videoEvent.key)
    print "videoEvent ";msg.GetInt();" data ";msg.GetData()
    if msg.GetInt() = 3 then
        SetVideoMetadata(videoPlayers, videoMetadata, videoEvent.key)
    else if msg.GetInt() = 8 then
        metadata = videoMetadata.Lookup(videoEvent.key)
        ' Video must play for no less than specified period minus 1000ms otherwise media error will be thrown
        if metadata.type = "video" AND metadata.stopped = false AND metadata.timer.TotalMilliseconds() < (metadata.duration - 1000) then
            videoEvent["eventType"] = 16
        endif
    ' When video loop mode enabled BS doesn't send ENDED (8) event automatically so we add Timecode event at moment
    ' of planned video end and such event is here mapped to ENDED (8) event  
    else if msg.GetInt() = 12 and msg.GetData() = 8 then
        videoEvent["eventType"] = 8
        videoPlayer.Pause()
    endif
    htmlWidget.PostJSMessage(videoEvent)
End Sub

Sub ProcessHardwareEvent(data, htmlWidget, serialPorts)
    hardwareEvent = CreateObject("roAssociativeArray")
    hardwareEvent["type"] = "hardware_event"
    hardwareEvent["data"] = data
    htmlWidget.PostJSMessage(hardwareEvent)
End Sub

Sub ProcessHtmlWidgetEvent(msg, msgPort, htmlWidget, videoModeProvider, supportedVideoModes, audioOutput, systemTime, screenRegistrySection, debugRegistrySection, videoPlayers, videoMetadata, touchScreen, serialPorts)
    eventData = msg.GetData()
    print "message=";eventData.message
    if type(eventData) = "roAssociativeArray" and type(eventData.reason) = "roString" then
        response = CreateObject("roAssociativeArray")
        if eventData.reason = "message" then
            if eventData.message.type = "system.reboot" then
                RebootSystem()
                response["succeeded"] = true
            endif
            if eventData.message.type = "application.restart" then
                RestartApplication()
                'RestartScript() 'soft restart of application
                response["succeeded"] = true
            endif
            if eventData.message.type = "display.power_on" then
                videoModeProvider.SetPowerSaveMode(false)
                videoModeProvider.EnableVideo(true)
                response["succeeded"] = true
            endif
            if eventData.message.type = "display.power_off" then
                videoModeProvider.SetPowerSaveMode(true)
                videoModeProvider.EnableVideo(false)
                response["succeeded"] = true
            endif
            if eventData.message.type = "screen.capture" then
                screenshotOptions = CreateObject("roAssociativeArray")
                screenshotOptions["filename"] = eventData.message.filePath
                screenshotOptions["width"] = eventData.message.width
                screenshotOptions["height"] = eventData.message.height
                screenshotOptions["filetype"] = eventData.message.fileType
                screenshotOptions["quality"] = eventData.message.quality
                screenshotOptions["rotation"] = eventData.message.rotation
                screenshotOptions["async"] = 0
                videoModeProvider.Screenshot(screenshotOptions)
                response["succeeded"] = true
            endif
            if eventData.message.type = "screen.set_brightness" then
                graphicsColorValues = CreateObject("roAssociativeArray")
                graphicsColorValues["brightness"] = eventData.message.brightness
                videoModeProvider.AdjustGraphicsColor(graphicsColorValues)
                response["succeeded"] = true
            endif
            if eventData.message.type = "audio.set_volume" then
                audioOutput["HDMI"].SetVolume(eventData.message.volume)
                audioOutput["Analog"].SetVolume(eventData.message.volume)
                audioOutput["SPDIF"].SetVolume(eventData.message.volume)
                audioOutput["USB"].SetVolume(eventData.message.volume)
                audioOutput["NONE"].SetVolume(eventData.message.volume)
                response["succeeded"] = true
            endif
            if eventData.message.type = "date_time.get_epoch" then
                currentDateTime = systemTime.GetUtcDateTime()
                response["seconds"] = currentDateTime.ToSecondsSinceEpoch()
                response["ms"] = currentDateTime.GetMillisecond()
                response["succeeded"] = true
            endif
            if eventData.message.type = "date_time.get_timezone" then
                utcDateTime = systemTime.GetUtcDateTime()
                localDateTime = systemTime.GetLocalDateTime()
                offsetInSeconds = utcDateTime.ToSecondsSinceEpoch() - localDateTime.ToSecondsSinceEpoch()
                response["offset"] = offsetInSeconds / 60
                response["year"] = localDateTime.GetYear()
                response["succeeded"] = true
            endif
            if eventData.message.type = "date_time.set_timezone" then
                systemTime.SetTimeZone(eventData.message.timezone)
                response["succeeded"] = true
            endif
            if eventData.message.type = "date_time.set_current_time" then
                currentDateTime = CreateObject("roDateTime")
                print "1"
                currentDateTime.FromSecondsSinceEpoch(strtoi(eventData.message.seconds))
                print "2"
                currentDateTime.SetMillisecond(strtoi(eventData.message.ms))
                print "3"
                systemTime.SetUtcDateTime(currentDateTime)
                print "4"
                response["succeeded"] = true
            endif
            if eventData.message.type = "screen.get_supported_video_modes" then
                response["list"] = FormatJson(supportedVideoModes, 1)
                response["succeeded"] = true
            endif
            if eventData.message.type = "screen.get_video_mode" then
            	videoMode = ParseJson(screenRegistrySection.Read("video_mode"))
            	response["width"] = videoMode["width"]
            	response["height"] = videoMode["height"]
                response["framerate"] = videoMode["framerate"]
            	response["succeeded"] = true
            endif
            if eventData.message.type = "screen.set_video_mode" then
            	response["succeeded"] = SaveVideoModeIfSupported(screenRegistrySection, supportedVideoModes, eventData.message.width, eventData.message.height, eventData.message.framerate)
            endif
            if eventData.message.type = "screen.set_orientation" then
                screenRegistrySection.Write("orientation", eventData.message.orientation)
                screenRegistrySection.Flush()
                transform = GetCurrentTransform(screenRegistrySection)
                if transform <> "identity" then
                    htmlWidget.SetTransform("identity")
                    sleep(1000)
                endif
                htmlWidget.SetTransform(transform)
                response["succeeded"] = true
            endif
            if eventData.message.type = "screen.get_orientation" then
                if screenRegistrySection.Exists("orientation") then
                    response["orientation"] = screenRegistrySection.Read("orientation")
                    response["transform"] = GetCurrentTransform(screenRegistrySection)
                else
                    response["orientation"] = "LANDSCAPE"
                    response["transform"] = "identity"
                endif
                response["succeeded"] = true
            endif
            if eventData.message.type = "screen.set_video_orientation" then
                screenRegistrySection.Write("video_orientation", eventData.message.orientation)
                screenRegistrySection.Flush()
                response["succeeded"] = true
            endif
            if eventData.message.type = "screen.get_video_orientation" then
                if screenRegistrySection.Exists("video_orientation") then
                    response["videoOrientation"] = screenRegistrySection.Read("video_orientation")
                else 
                    response["videoOrientation"] = "LANDSCAPE"
                endif
                response["succeeded"] = true
            endif
            if eventData.message.type = "screen.clear_video_orientation" then
                screenRegistrySection.Delete("video_orientation")
                screenRegistrySection.Flush()
                response["succeeded"] = true
            endif
            if eventData.message.type = "debug.get_debug" then
                response["enabled"] = debugRegistrySection.Read("native")
                response["succeeded"] = true
            endif
            if eventData.message.type = "debug.set_debug" then
                debugRegistrySection.Write("native", eventData.message.enabled)
                debugRegistrySection.Flush()
                response["succeeded"] = true
            endif
            if eventData.message.type = "video.init_player" then
                videoPlayerKey = eventData.message.key
                videoPlayer = CreateObject("roVideoPlayer")
                videoPlayer.SetPort(msgPort)
                videoPlayer.SetUserData(videoPlayerKey)
                videoPlayer.SetViewMode("LetterboxedAndCentered")
                videoPlayers.AddReplace(videoPlayerKey, videoPlayer)
                response["succeeded"] = true
            endif
            if eventData.message.type = "video.prepare" then
                videoPlayerKey = eventData.message.key
                videoPlayer = videoPlayers.Lookup(videoPlayerKey)
                PrepareMedia(screenRegistrySection, videoModeProvider, videoPlayer, eventData)
                SetMetadataType(videoMetadata, videoPlayerKey, eventData.message.vtype)

                if eventData.message.vtype = "video" then
                    videoFilePath = eventData.message.filePath
                    videoVolume = strtoi(eventData.message.volume)
                    PrepareVideo(videoPlayer, videoFilePath, videoVolume)
                else if eventData.message.vtype = "stream" then
                    streamUri = eventData.message.uri
                    PrepareStream(videoPlayer, streamUri)
                endif
                response["succeeded"] = true
            endif
            if eventData.message.type = "video.play" then
                videoPlayerKey = eventData.message.key
                videoPlayer = videoPlayers.Lookup(videoPlayerKey)
                AppendEndedEvent(videoPlayer)
                videoPlayer.Play()
                response["succeeded"] = true
            endif
            if eventData.message.type = "video.prepare_and_play" then
                videoPlayerKey = eventData.message.key
                videoPlayer = videoPlayers.Lookup(videoPlayerKey)
                PrepareMedia(screenRegistrySection, videoModeProvider, videoPlayer, eventData)
                SetMetadataType(videoMetadata, videoPlayerKey, eventData.message.vtype)
                AppendEndedEvent(videoPlayer)

                if eventData.message.vtype = "video" then
                    videoFilePath = eventData.message.filePath
                    videoVolume = strtoi(eventData.message.volume)
                    PlayVideo(videoPlayer, videoFilePath, videoVolume)
                else if eventData.message.vtype = "stream" then
                    streamUri = eventData.message.uri
                    PlayStream(videoPlayer, streamUri)
                else if eventData.message.vtype = "hdmi" then
                    PlayHDMI(videoPlayer)
                endif
                response["succeeded"] = true
            endif
            if eventData.message.type = "video.get_duration" then
                videoPlayerKey = eventData.message.key
                videoPlayer = videoPlayers.Lookup(videoPlayerKey)
                response["duration"] = videoPlayer.GetDuration()
                response["succeeded"] = true
            endif
            if eventData.message.type = "video.stop" then
                videoPlayerKey = eventData.message.key
                videoPlayer = videoPlayers.Lookup(videoPlayerKey)
                videoPlayer.SetLoopMode(false)
                videoPlayer.ClearEvents()
                SetMetadataStopped(videoMetadata, videoPlayerKey)
                if eventData.message.clear = "1" then
                    videoPlayer.StopClear()
                else
                    videoPlayer.Stop()
                endif
                response["succeeded"] = true
            endif
            if eventData.message.type = "video.pause" then
                videoPlayerKey = eventData.message.key
                videoPlayer = videoPlayers.Lookup(videoPlayerKey)
                videoPlayer.Pause()
                response["succeeded"] = true
            endif
            if eventData.message.type = "video.resume" then
                videoPlayerKey = eventData.message.key
                videoPlayer = videoPlayers.Lookup(videoPlayerKey)
                videoPlayer.Resume()
                response["succeeded"] = true
            endif
            if eventData.message.type = "network.set_manual" then
                networkInterfaceName = eventData.message.interface
                ipAddress = eventData.message.ip
                netmask = eventData.message.netmask
                gateway = eventData.message.gateway
                dnsServers = eventData.message.dns
                networkInterface = CreateObject("roNetworkConfiguration", networkInterfaceName)
                if networkInterface <> invalid then
                    networkInterface.SetIP4Address(ipAddress)
                    networkInterface.SetIP4Netmask(netmask)
                    networkInterface.SetIP4Gateway(gateway)
                    networkInterface.SetDNSServers(dnsServers)
                    response["succeeded"] = networkInterface.Apply()
                else
                    response["succeeded"] = false
                endif
            endif
            if eventData.message.type = "network.set_dhcp" then
                networkInterfaceName = eventData.message.interface
                networkInterface = CreateObject("roNetworkConfiguration", networkInterfaceName)
                if networkInterface <> invalid then
                    networkInterface.SetDHCP()
                    response["succeeded"] = networkInterface.Apply()
                else
                    response["succeeded"] = false
                endif
            endif
            if eventData.message.type = "network.get_ntp" then
                networkInterfaceName = eventData.message.interface
                networkInterface = CreateObject("roNetworkConfiguration", networkInterfaceName)
                if networkInterface <> invalid then
                    response["ntp"] = networkInterface.GetTimeServer()
                else
                    response["ntp"] = ""
                endif
                response["succeeded"] = true
            endif
            if eventData.message.type = "network.set_ntp" then
                networkInterfaceName = eventData.message.interface
                ntpServer = eventData.message.ntp
                networkInterface = CreateObject("roNetworkConfiguration", networkInterfaceName)
                if networkInterface <> invalid then
                    networkInterface.SetTimeServer(ntpServer)
                    response["succeeded"] = networkInterface.Apply()
                else
                    response["succeeded"] = false
                endif
            endif
            if eventData.message.type = "network.get_wifi_info" then
                networkInterface = CreateObject("roNetworkConfiguration", 1)
                if networkInterface <> invalid then
                    response["ssid"] = networkInterface.GetCurrentConfig().wifi_essid
                    response["strength"] = networkInterface.GetCurrentConfig().wifi_signal
                else
                    response["ssid"] = ""
                    response["strength"] = 0
                endif
                response["succeeded"] = true
            endif
            if eventData.message.type = "monitors.get_list" then
                edidHdmiList = videoModeProvider.GetEdidIdentity(true)
                edidVgaList = videoModeProvider.GetEdidIdentity(false)
                list = CreateObject("roAssociativeArray")
                if not(edidHdmiList = invalid) then
                    list.hdmi = edidHdmiList
                end if
                if not(edidVgaList = invalid) then
                    list.vga = edidVgaList
                end if
                response["list"] = FormatJson(list, 1)
                response["succeeded"] = true
            end if
            if eventData.message.type = "system.device_info" then
                deviceInfo = CreateObject("roDeviceInfo")
                if eventData.message.infoType = "cpu" then
                    response["data"] = deviceInfo.GetLoadStatistics({item: "loadavg"})
                else if eventData.message.infoType = "memory" then
                    loadStatistics = deviceInfo.GetLoadStatistics({item: "meminfo"})
                    response["data"] = loadStatistics.Trim()
                else if eventData.message.infoType = "usb" then
                    usbTopology = deviceInfo.GetUSBTopology({rebuild: true})
                    response["data"] = usbTopology
                endif
                response["succeeded"] = true
            end if
            if eventData.message.type = "proxy.get_info" then
                networkInterfaceName = eventData.message.interface
                networkInterface = CreateObject("roNetworkConfiguration", networkInterfaceName)
                if networkInterface <> invalid then
                    response["proxy"] = networkInterface.GetCurrentConfig().current_proxy
                else
                    response["proxy"] = ""
                endif
                response["succeeded"] = true
            end if
            if eventData.message.type = "proxy.set_info" then
                networkInterfaceName = eventData.message.interface
                proxyUrl = eventData.message.proxyUrl
                networkInterface = CreateObject("roNetworkConfiguration", networkInterfaceName)
                if networkInterface <> invalid then
                    networkInterface.SetProxy(proxyUrl)
                    response["succeeded"] = networkInterface.Apply()
                else
                    response["succeeded"] = false
                endif
            end if
            if eventData.message.type = "hardware.serial_open_port" then
                device = eventData.message.device
                baudRate = StrToI(eventData.message.baudRate)
                mode = eventData.message.mode
                serialPort = CreateObject("roSerialPort", StrToI(device), baudRate)
                if serialPort <> Invalid then
                    serialPort.SetLineEventPort(msgPort)
                    serialPort.SetBaudRate(baudRate)
                    serialPort.SetMode(mode)
                    serialPorts.AddReplace(device, serialPort)
                    response["succeeded"] = true
                else
                    response["data"] = "Failed to open port"
                    response["succeeded"] = false
                endif
            end if
            if eventData.message.type = "hardware.serial_write_message" then
                device = eventData.message.device
                message = eventData.message.message
                serialPort = serialPorts.Lookup(device)
                if serialPort <> Invalid then
                    serialPort.SendLine(message)
                    response["device"] = device
                    response["message"] = message
                    response["succeeded"] = true
                else
                    response["device"] = device
                    response["message"] = message
                    response["data"] = "Error, port not found"
                    response["succeeded"] = false
                endif
            end if
            if eventData.message.type = "hardware.serial_close_port" then
                device = eventData.message.device
                serialPort = serialPorts.Lookup(device)
                if serialPort <> Invalid then
                    serialPort.Flush()
                    serialPorts.Delete(device)
                    response["succeeded"] = true
                else
                    response["data"] = "Failed to close serial port"
                    response["succeeded"] = false
                endif
            end if 
            response["type"] = "response"
            response["invocationUid"] = eventData.message.invocationUid
            htmlWidget.PostJSMessage(response)
            ' BrightSign will be rebooted after sending response when the resolution changed
            if eventData.message.type = "screen.set_video_mode" AND response["succeeded"] = true then
                RebootSystem()
            end if            
        endif
        if eventData.reason = "load-error" then
            print "message = ";eventData.message
        endif
    endif
End Sub
